/******************************************************************************
 *
 * Copyright:
 *    (C) 2007 Embedded Artists AB
 *
 * Description:
 *    Sample application that demonstrates how to communicate
 *    with the touch screen controller.
 *
 *****************************************************************************/

#include <general.h>
#include <printf_P.h>
#include <ea_init.h>
#include <lpc246x.h>

#define CS_PIN        0x00010000        //P0.16
#define ACTIVATE_CS   IOCLR0 = CS_PIN
#define DEACTIVATE_CS IOSET0 = CS_PIN


/******************************************************************************
** Function name:		mdelay
**
** Descriptions:		
**
** parameters:			delay length
** Returned value:		None
** 
******************************************************************************/
void mDelay( unsigned int delayInMs )
{
  /*
   * setup timer #1 for delay
   */
  T1TCR = 0x02;          //stop and reset timer
  T1PR  = 0x00;          //set prescaler to zero
  T1MR0 = delayInMs * (Fpclk / 1000);
  T1IR  = 0xff;          //reset all interrrupt flags
  T1MCR = 0x04;          //stop timer on match
  T1TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (T1TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *
 ****************************************************************************/
static unsigned char
sendSpi(unsigned char indata)
{
	unsigned int failsafe;
	
  S0SPDR = indata;
  failsafe = 0;
  while(((S0SPSR & 0x80) == 0) && (failsafe < 5000))
    failsafe++;
  
  if (failsafe >= 5000)
  {
    S0SPCCR = 57;
    S0SPCR  = 0x38;
  }
  return S0SPDR;
}


/*****************************************************************************
 *
 * Description:
 *    The first function to execute 
 *
 ****************************************************************************/
int
main(void)
{
  eaInit();
  printf("\n***************************************************");
  printf("\n*                                                 *");
  printf("\n* Sample program for LPC2468 OEM Board...         *");
  printf("\n* (C) Embedded Artists 2006-2007                  *");
  printf("\n*                                                 *");
  printf("\n* QVGA LCD example code                           *");
  printf("\n*                                                 *");
  printf("\n***************************************************\n");

  //init SPI #0
  PINSEL0 |= 0xc0000000;
  PINSEL1 |= 0x0000003c;
  IODIR0  |= CS_PIN;
  IOSET0   = CS_PIN;
  S0SPCCR = 57;    
  S0SPCR  = 0x38;

  while ( 1 )
  {
    tU32 Xresult;
    tU32 Yresult;
    tU32 Z1result;
    tU32 Z2result;
    tU32 pressure;

    //Read X value
    ACTIVATE_CS;
    sendSpi(0x83 | 0x50 | 0x00);
    Xresult  = ((tU32)sendSpi(0x00)) << 8;
    Xresult |= (tU32)sendSpi(0x00);
    Xresult >>= 3;
    DEACTIVATE_CS;

    //Read Y value
    ACTIVATE_CS;
    sendSpi(0x83 | 0x10 | 0x00);
    Yresult  = ((tU32)sendSpi(0x00)) << 8;
    Yresult |= (tU32)sendSpi(0x00);
    Yresult >>= 3;
    DEACTIVATE_CS;

    //Read Z1 value
    ACTIVATE_CS;
    sendSpi(0x83 | 0x30 | 0x00);
    Z1result  = ((tU32)sendSpi(0x00)) << 8;
    Z1result |= (tU32)sendSpi(0x00);
    Z1result >>= 3;
    DEACTIVATE_CS;

    //Read Z2 value
    ACTIVATE_CS;
    sendSpi(0x83 | 0x40 | 0x00);
    Z2result  = ((tU32)sendSpi(0x00)) << 8;
    Z2result |= (tU32)sendSpi(0x00);
    Z2result >>= 3;
    DEACTIVATE_CS;

    //Calculate pressure
    pressure = (Xresult*Z2result - Z1result) / Z1result;

    //Print out result
    printf("\nX=%d     Y=%d     Pressure=%d", Xresult, Yresult, pressure);

    if ((pressure != 0) && (pressure < 40000))
      printf("   -------  pressed!!!");

    mDelay(50);
  }
}
